######################## Chapter 1 Script #################
###########################################################

#Citation for R
citation()

#########################
# 1.2 Starting up R #####
#########################

#Set working directory to Chapter1 folder
setwd("C:/Docs/RCompanion/Chapter1/")

#Save current workspace
save.image("C:/Docs/RCompanion/Chapter1/scratch.RData")

#Load an earlier saved workspace
load("C:/Docs/RCompanion/Chapter1/scratch.RData")

############################
# 1.3 Searching for Help ###
############################

#Using the help.search command
help.search("arithmetic")

#Using the help command
help("+",package=base)

#Error messades in using the help command
help(+,package=base)

help(x,package=base)

#Error in using the help.search command
help.search("brown-forsyth test")

#Using help.search to interpret message terminology
help.search("fuzzy matching")

#Help on package base
help(package=base)
library(help=base)
help(base,package=base)
help(base,package="base")

#Keyword search example
help.search("anova")

###########################################
# 1.4 Managing Objects in the Workspace ###
###########################################

#Data for illustrations of Objects in R workspace
x<-1:25; y<-2*x+3; z<-3*x-2*y
pairs<-data.frame("x"=x,"y"=y)
triples<-data.frame("x"=x,"y"=x,"z"=x)

#List all objects in workspace
ls()

#Remove selected objects from workspace
rm("x","y","z")

#Saving objects
dump("triples",
         "C:/Docs/RCompanion/Chapter1/Data/triples.R")
dump(c("pairs","triples"),
	"C:/Docs/RCompanion/Chapter1/Data/pairsntriples.R")
dump(list=ls(all=TRUE),
	"C:/Docs/RCompanion/Chapter1/Data/alldata.R")

#Removing all objects from the workspace
rm(list=ls(all=TRUE))

##################################################
# 1.5 Installing and Loading Packages from CRAN ##
##################################################

#List packages loaded in the workspace
search()

#List packages installed on computer
library()
installed.packages()

#Loading a package into the active memory
library(car)
require(faraway)

#Detaching packages
detach(package:car)
detach(package:faraway)

#Warning messages when loading and detaching packages
library(HH)
detach(package:HH); detach(package:lattice); detach(package:grid)
detach(package:multcomp); detach(package:mvtnorm); detach(package:survival)
detach(package:splines); detach(package:leaps)

############################
# 1.6 Attaching R Objects ##
############################

#Clear workspace
rm(list=ls(all=T))

#Upload objects
source("C:/Docs/RCompanion/Chapter1/Data/pairsntriples.R")
ls()

#Names of object contents
names(pairs)
names(triples)
x

#Attaching the contents of an object to the workspace
attach(pairs)
x

#Determining which objects are attached
search()

#Attaching competing objects
attach(triples)

#Detaching objects
detach(pairs); detach(triples)

########################################
# 1.7 Saving Graphics Images from R ###
########################################

#Exporting graphics images
win.graph(width=3.5,height=3)
par(ps=10,cex=.75)
plot(pairs)

###########################################
# 1.8 Viewing and Saving Session History ##
###########################################

#Session history
history(max.show=25)
history(max.show=Inf)
savehistory("C:/Docs/RCompanion/Chapter1/Chapter1history.txt")
savehistory()

#########################################
# 1.9 Citing R and Packages from CRAN ###
#########################################

citation()

citation("car")

##############################
# 1.10 The R Script Editor ###
##############################

#Running lines of script
win.graph(width=3.5,height=3); par(ps=10,cex=.75)
plot(pairs)

#Writing and running multi-line commands
pnorm(21.0,mean=17.5,sd=sqrt(3))-
                pnorm(17.5,mean=17.5,sd=sqrt(3))

pnorm(21.0,mean=17.5,sd=sqrt(3))
                -pnorm(17.5,mean=17.5,sd=sqrt(3))

(pnorm(21.0,mean=17.5,sd=sqrt(3))
                -pnorm(17.5,mean=17.5,sd=sqrt(3)))

