####################### Chapter 14 Script #################
############################################################

#CAUTION: Either restart R or be sure to reset
#the baseline treatment and contrast type to default

#Clear workspace
rm(list=ls(all=T))

################################################################
#Execute this command to duplicate first part of this Chapter ##
################################################################

#Data for additive model
source("C:/Docs/RCompanion/Chapter14/Data/Data14x01.R")

###################################
# 14.2 Exploratory data analysis ##
###################################

#Data summary
summary(TwoWayAdd)

#............
# Fig 14.1 ..
#............

win.graph(width=6,height=3.5); par(mfrow=c(1,2),cex=.75,ps=10)
with(TwoWayAdd,interaction.plot(A,B,y))
with(TwoWayAdd,interaction.plot(B,A,y))
par(mfrow=c(1,1))

#............
# Fig 14.2 ..
#............

attach(TwoWayAdd)
win.graph(width=6,height=6); par(mfrow=c(2,2),ps=10,cex=.75)
stripchart(y~A,xlab="y",method="jitter",ylab="A",las=1)
stripchart(y~B,xlab="y",method="jitter",ylab="B",las=1)
plot(y~A,ylab="y",xlab="A",horizontal=T,las=1)
plot(y~B,ylab="y",xlab="B",horizontal=T,las=1)
par(mfrow=c(1,1))
detach(TwoWayAdd)

########################################
# 14.3 Model and Construction and Fit ##
########################################

#Default treatment coding model

#Nonadditive model
summary(aov(y~A+B+A:B,TwoWayAdd))

#Additive model
TwoWayAdd.mod<-aov(y~A+B,TwoWayAdd)
TwoWayAdd.sum<-summary.lm(TwoWayAdd.mod)

#Get ANOVA table
summary(TwoWayAdd.mod)

###############################
# 14.4 Graphical diagnostics ##
###############################

e<-residuals(TwoWayAdd.mod)
y.hat<-fitted.values(TwoWayAdd.mod)
r<-e/(TwoWayAdd.sum$sigma*sqrt(1-hatvalues(TwoWayAdd.mod)))
d<-rstudent(TwoWayAdd.mod)

#............
# Fig 14.3 ..
#............

#Residual analysis for error term assumptions

win.graph(width=6,height=6.5); par(mfrow=c(2,2),cex=.75,ps=10)
with(TwoWayAdd,stripchart(r~A,method="jitter",
  xlab="A",ylab=expression(hat(r)),
	sub="(a)",vertical=T,las=1))
with(TwoWayAdd,stripchart(r~B,method="jitter",
  xlab="B",ylab=expression(hat(r)),
	sub="(b)",vertical=T,las=1))
plot(r~y.hat,xlab=expression(hat(y)),ylab=expression(hat(r)),
	sub="(c)",las=1)
qqnorm(r,main=NULL,las=1,sub="(d)");abline(a=0,b=1,lty=3)
par(mfrow=c(1,1))

#............
# Fig 14.4 ..
#............

#Outliers analysis
win.graph(width=3,height=3.5); par(cex=.75,ps=10)
plot(d,ylab=expression(hat(d)))
m<-ceiling(.05*length(d));df<-TwoWayAdd.mod$df.residual-1
cv<-qt(0.05/(2*m),df,lower.tail=F)
abline(h=c(-cv,0,cv),lty=c(3,1,3))
identify(d)

#....Numerical tests for constant variance assumption
# Brown-Forsyth test - using Anova.bf.test
source("C:/Docs/RCompanion/Chapter11/Functions/AnovaBFFTest.R")

with(TwoWayAdd,Anova.bf.Ftest(e,A))
with(TwoWayAdd,Anova.bf.Ftest(e,B))

# Brown-Forsyth test - using hov
library(HH)
hov(e~A,TwoWayAdd)
hov(e~B,TwoWayAdd)
detach(package:HH);detach(package:lattice)
detach(package:grid);detach(package:multcomp)
detach(package:mvtnorm);detach(package:survival)
detach(package:splines);detach(package:leaps)

#....Numerical tests for normality assumption
#QQ normal probability correlation coefficient test
source("C:/Docs/RCompanion/Chapter6/Functions/qqcortest.R")
qq.cortest(r,0.05)

#Shapiro-Wilk test
shapiro.test(r)

#Influence analysis
inf.tests<-data.frame(influence.measures(TwoWayAdd.mod)[2])
inf.tests[47,]

###################################################
# 14.5 Pairwise Comparisons of Treatment Effects ##
###################################################

#...................................
# 14.5.1 With a control treatment ..
#...................................

TwoWayAdd.sum$coefficients

#......................................
# 14.5.2 Without a control treatment ..
#......................................

#Tukey's HSD procedure
TukeyHSD(aov(y~A+B,TwoWayAdd),"A",ordered=T)
TukeyHSD(aov(y~B+A,TwoWayAdd),"B",ordered=T)

#Compare differences obtained using differently arranged formula
TukeyHSD(aov(y~A+B,TwoWayAdd),ordered=T)
TukeyHSD(aov(y~B+A,TwoWayAdd),ordered=T)


#Bonferroni's procedure
#..........Factor A
#For convenience
attach(TwoWayAdd)
#Initialize counters
n<-length(y);I<-length(levels(A));J<-length(levels(B))
g<-choose(I,2)
#Get cell sizes and means
nA<-tapply(y,A,length);means<-tapply(y,A,mean)
detach(TwoWayAdd)
#Initialize storage variables
Pair<-character(g);Diff<-numeric(g)
S.er<-numeric(g)
Tv<-numeric(g);lwr<-numeric(g)
upr<-numeric(g);Pv<-numeric(g)
#Get critical value for intervals
cv<-qt(.05/(2*g),n-I-J+1,lower.tail=F)
#Initialize test counter and begin for loops
k<-1
for (l in 1:(I-1)){
	for (m in (l+1):I){
		#Perform calculations
		Pair[k]<-paste("A",l," vs. ","A",m,sep="")
		Diff[k]<-means[l]-means[m]
		S.er[k]<-TwoWayAdd.sum$sigma*sqrt(1/nA[l]+1/nA[m])
		lwr[k]<-round(Diff[k]-cv*S.er[k],4)
		upr[k]<-round(Diff[k]+cv*S.er[k],4)
		Tv[k]<-abs(Diff[k])/S.er[k]
		Pv[k]<-round(2*g*pt(Tv[k],n-I-J+1,lower.tail=F),3)
		Pv[k]<-ifelse(Pv[k]>1,1,Pv[k]);k<-k+1}}
#Store results
ResultsA<-data.frame(Diff,S.er,lwr,upr,Pv,row.names=Pair)


#..........Factor B
#For convenience
attach(TwoWayAdd)
#Initialize counters
n<-length(y);I<-length(levels(A));J<-length(levels(B))
g<-choose(J,2)
#Get cell sizes and means
nB<-tapply(y,B,length);means<-tapply(y,B,mean)
detach(TwoWayAdd)
#Initialize storage variables
Pair<-character(g);Diff<-numeric(g)
S.er<-numeric(g)
Tv<-numeric(g);lwr<-numeric(g)
upr<-numeric(g);Pv<-numeric(g)
#Get critical value for intervals
cv<-qt(.05/(2*g),n-I-J+1,lower.tail=F)
#Initialize test counter and begin for loops
k<-1
for (l in 1:(J-1)){
	for (m in (l+1):J){
		#Perform calculations
		Pair[k]<-paste("B",l," vs. ","B",m,sep="")
		Diff[k]<-means[l]-means[m]
		S.er[k]<-TwoWayAdd.sum$sigma*sqrt(1/nB[l]+1/nB[m])
		lwr[k]<-round(Diff[k]-cv*S.er[k],4)
		upr[k]<-round(Diff[k]+cv*S.er[k],4)
		Tv[k]<-abs(Diff[k])/S.er[k]
		Pv[k]<-round(2*g*pt(Tv[k],n-I-J+1,lower.tail=F),3)
		Pv[k]<-ifelse(Pv[k]>1,1,Pv[k]);k<-k+1}}
#Store results
ResultsB<-data.frame(Diff,S.er,lwr,upr,Pv,row.names=Pair)
list("A"=ResultsA,"B"=ResultsB)

######################################################
# 14.6 What if interaction effects are significant? ##
######################################################

source("C:/Docs/RCompanion/Chapter14/Data/Data14x02.R")

#............
# Fig 14.5 ..
#............

win.graph(width=6,height=3.5); par(mfrow=c(1,2),cex=.75,ps=10)
with(TwoWayInt,interaction.plot(A,B,y))
with(TwoWayInt,interaction.plot(B,A,y))
par(mfrow=c(1,1))

#Test interaction effects
summary(aov(y~A+B+A:B,TwoWayInt))

#Create and populate the variable AB
#For convenience
attach(TwoWayInt);
#Initialize the cell variable
AB<-character(length(y))
#Store values in AB
for (i in 1:length(y)){AB[i]<-paste(A[i],B[i],sep="")}
#Create a new data frame
IntMod.dat<-data.frame(y,AB); rm(AB)
#Define AB as a factor, to be safe
IntMod.dat$AB<-factor(IntMod.dat$AB)
detach(TwoWayInt)

#Test treatment effects for the new model
summary(aov(y~AB,IntMod.dat))

#Perform Tukey's HSD and plot
tests<-TukeyHSD(aov(y~AB,IntMod.dat))
plot(tests)

#Sift through Tukey's test results
#Determine number of insignificant differences
attach(tests); j<-0
for (i in 1:66){
	if (AB[i,4]>=0.05) {j<-j+1}}
detach(tests);j

#Find pairs that are not significantly different
attach(tests)
H.Pairs<-NULL;
for (i in 1:66){
     if (AB[i,4]>=0.05){
	H.Pairs<-c(H.Pairs,row.names(AB)[i])}}
detach(tests);H.Pairs

#########################
# 14.8 For the curious ##
#########################

#Scheffe's F-tests
#for convenience
attach(TwoWayAdd)
#Gather various lengths
n<-length(y); I<-length(levels(A))
J<-length(levels(B))
#Obtain number of pairwise comparisons
gA<-choose(I,2);gB<-choose(J,2)
#Obtain cell sizes
nA<-tapply(y,A,length);nB<-tapply(y,B,length)
#Obtain cell means
meansA<-tapply(y,A,mean);meansB<-tapply(y,B,mean)
detach(TwoWayAdd)
#Initialize various variables that will be needed
PairA<-character(gA);PairB<-character(gB)
DiffA<-numeric(gA);DiffB<-numeric(gB)
S.erA<-numeric(gA);S.erB<-numeric(gB)
TvA<-numeric(gA);TvB<-numeric(gB)
lwrA<-numeric(gA);lwrB<-numeric(gB)
uprA<-numeric(gA);uprB<-numeric(gB)
PvA<-numeric(gA);PvB<-numeric(gB)
#Obtain "critical" values
cvA<-sqrt((I-1)*qf(.05,(I-1),n-I-J+1,lower.tail=F))
cvB<-sqrt((J-1)*qf(.05,(J-1),n-I-J+1,lower.tail=F))
#initialize comparison counter
k<-1
#Start computing details for factor A
for (l in 1:2){for (m in (l+1):3){
   #Identify pair, obtain signed difference in means and standard error
   PairA[k]<-paste("A",l," vs. ","A",m,sep="")
   DiffA[k]<-meansA[l]-meansA[m]
   S.erA[k]<-TwoWayAdd.sum$sigma*sqrt(1/nA[l]+1/nA[m])
   #Compute lower and upper bounds and test values
   lwrA[k]<-DiffA[k]-cvA*S.erA[k]
   uprA[k]<-DiffA[k]+cvA*S.erA[k]
   TvA[k]<-(DiffA[k])^2/((I-1)*S.erA[k]^2)
   #Get p-values and close the loop
   PvA[k]<-round(pf(TvA[k],I-1,n-I-J+1,lower.tail=F),3)
   PvA[k]<-ifelse(PvA[k]>1,1,PvA[k])
   k<-k+1}}
#Store the results
ScheffeA<-data.frame(cbind(DiffA,S.erA,lwrA,uprA,PvA),
                                       row.names=PairA)
#Repeat the process for factor B
k<-1
for (l in 1:3){for (m in (l+1):4){
PairB[k]<-paste("B",l," vs. ","B",m,sep="")
DiffB[k]<-meansB[l]-meansB[m]
S.erB[k]<-TwoWayAdd.sum$sigma*sqrt(1/nB[l]+1/nB[m])
lwrB[k]<-DiffB[k]-cvA*S.erB[k]
uprB[k]<-DiffB[k]+cvB*S.erB[k]
TvB[k]<-(DiffB[k])^2/((J-1)*S.erB[k]^2)
PvB[k]<-round(pf(TvB[k],J-1,n-I-J+1,lower.tail=F),3)
PvB[k]<-ifelse(PvB[k]>1,1,PvB[k])
k<-k+1}}
ScheffeB<-data.frame(cbind(DiffB,S.erB,lwrB,uprB,PvB),
                                       row.names=PairB)
#Output the results
list("A"=round(ScheffeA,3),"B"=round(ScheffeB,3))
