####################### Chapter 15 Script #################
############################################################

#CAUTION: Either restart R or be sure to reset
#the baseline treatment and contrast type to default

#Clear workspace
rm(list=ls(all=T))

#######################################
# 15.3 Issues Specific to Covariates ##
#######################################

#################################################
#Execute the command to duplicate this section ##
#################################################

source("C:/Docs/RCompanion/Chapter15/Data/Data15x01.R")

#............
# Fig 15.1 ..
#............

win.graph(width=3,height=3.5);par(ps=10,cex=.75)
plot(y~x,CovToBlock,pch=unclass(Tr),main=NULL,las=1)

#Convert covariate data into block levels
#Determine the lower and upperbounds for the covariate
with(CovToBlock,range(x))
#Obtain the partition
bpts<-c(0,0.6,1.4,2)
#Create the classes
blocks<-with(CovToBlock,cut(x,breaks=bpts,include.lowest=T,right=F))
#Store the results
BlockData<-with(CovToBlock,data.frame(y,Tr,blocks))


BlockData

#########################
# Data Generation Code ##
#########################

#Define the mean response function
f<-function(z){15-(z-2)^4}
#Set cell size, initialize variables and "observation" counter
n<-10; Tr<-integer(3*n); y<-numeric(3*n)
x<-numeric(3*n); k<-1
#Start outer for-loop to generate data
for (j in 1:3){
   #Get some x-values and set a treatment mean
   xx<-round(runif(n,0,2),2); mu<-2.5*j
   #Get some random error terms
   e<-round(rnorm(n,mean=mu,sd=.25),1)
   #Start inner for-loop
   for (i in 1:n){
      #Assign factor levels and obtain "observed" responses
      Tr[k]<-j; y[k]<-round(f(xx[[i]])+e[i],2)
      #Save x-values and update "observation" counter
      x[k]<-xx[i]; k<-k+1}}
#Store the data
CovToBlock<-data.frame(y,x,Tr)
#Define Tr as a factor
CovToBlock$Tr<-factor(CovToBlock$Tr)

#Obtain a scatterplot, just to be sure
win.graph(width=3,height=3.5);par(ps=10,cex=.75)
plot(y~x,CovToBlock,pch=unclass(Tr),main=NULL,las=1)

#Can be saved using the dump function



