######################## Chapter 3 Script #################
###########################################################

################################################
# 3.2 Naming and Initializing Data Structures ##
################################################

#What happens when an undefined object is referenced

ice

#Populating a numerical variable and recalling its contents
ice<-c(1,1,.8,.55,.25,.05,0,0,0,.1,.75,.95)
ice

#Populating a character variable and recalling its contents
month<-c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug",
"Sep","Oct","Nov","Dec")
month

#Listing objects in workspace
ls()

#Anappropriate combining of variables
lake.data<-c(ice,month)
lake.data

cbind(ice,month)
rbind(ice,month)

#Using a data frame to collect data
lake.data<-data.frame("ice"=ice,"month"=month)
lake.data

lake.data<-data.frame(ice,month)
lake.data

#Contents of a data frame
names(lake.data)

#Renaming variables in a data frame
names(lake.data)<-c("Ice.Cover","Month.of.Year")
names(lake.data)

######################################################
# 3.3 Classification of Data within Data Structures ##
######################################################

mode(ice)
mode(month)
mode(c(ice,month))
mode(cbind(ice,month))
mode(lake.data)

#####################################
# 3.4 Basics with Univariate Data ###
#####################################

#Clear workspace
rm(list=ls(all=T))

#Create a couple of datasets
Evens<-c(2,4,6,8,10,12)
Colors<-c("Red","Blue","Green")

#........................................................
# 3.4.1 Accessing the Contents of Univariate Datasets ...
#........................................................

length(Evens)
Evens
Evens[5]
Evens[-3]
Evens[4:6]
Evens[-(4:6)]
Evens[seq(1,5,2)]
Evens[Evens>4]

Colors
Colors<-c(Colors,"Pink")
Colors[2]<-"Brown"
Colors

#......................................
# 3.4.2 Sorting Univariate datasets ...
#......................................

#Create a dataset
x<-c(24,23,11,24,19,23,12); x

#Sort in ascending order
sort(x,decreasing=F)

#Get ranks of entries
rank(x,ties.method="first")

#obtain order of indices
order(x)

#Sort by providing ordered indices
x[order(x)]

#................................................
# 3.4.3 Computations with Univariate datasets ...
#................................................

rm(list=ls(all=T))

#Create datasets
u<-c(1,3,-1); v<-c(-1,0,1)

#Perform linear combinations
u+v
u-v
2*u-3*v

#Perform multiplication and division
u*v
u/v
v/u

#Use in mathematical functions
round(exp(u),2)
log(v)
u^v

#Obtain sums and products of entries
sum(u)
sum(v^2)
prod(u)

#Explore ways in which mistakes can creep in
u<-c(u,2)
u+v

u<-c(3,u,1)
u+v

#....................................
# 3.4.4 Special Vector operations ...
#....................................

#Create vectors
x<-as.vector(c(1,3,2,4))
y<-as.vector(c(-3,2,-1,3))

#Look at contents in different ways
x
as.matrix(x)
t(x)

#Indirect dot product
sum(x*y)

#Direct dot product
crossprod(x,y)
tcrossprod(x,y)

#Alternative views of the above products 
tcrossprod(y,x)
crossprod(y,x)

#Matrix operators
x%*%y
x%o%y

#.................................................
# 3.4.5 Importing datasets from external files ...
#.................................................

#Clear workspace
rm(list=ls(all=T))

#Read data from a csv file
SmallSet<-read.csv("C:/Docs/RCompanion/Chapter3/Data/Data03x01.csv",header=T)

#Read data from a text file
SmallSet<-read.table("C:/Docs/RCompanion/Chapter3/Data/Data03x01.txt",header=T)

#See contents
SmallSet

#List objects in workspace
ls()

#Names of objects in SmallSet
names(SmallSet)

#Mode of SmallSet
mode(SmallSet)

#Explore contents of SmallSet
#...By columns
SmallSet[,1]
#...By variable
SmallSet$x
#...By attaching
attach(SmallSet); x; detach(SmallSet)
#...Using the with function
with(SmallSet,x)

#To save SmallSet
dump("SmallSet",
   "C:/Docs/RCompanion/Chapter3/Data/Data03x01.R")
rm(list=ls(all=T));ls()

#To recall SmallSet
source("C:/Docs/RCompanion/Chapter3/Data/Data03x01.R")
ls()

#Extracting contents for use in a new variable
x<-SmallSet$x; x

#Clear workspace
rm(list=ls(all=T))

######################################
# 3.5 Basics with Multivariate Data ##
######################################

#Read data from a text file
CatSet<-read.table(
 "C:/Docs/RCompanion/Chapter3/Data/Data03x02.txt",header=T)

#Save in a source file
dump("CatSet",
   "C:/Docs/RCompanion/Chapter3/Data/Data03x02.R")

#.........................................................
# 3.5.1 Accessing the contents of multivariate datasets ..
#.........................................................

#Look at all contents
CatSet

#Names of enclosed variables
names(CatSet)

#Look at contents of each variable
CatSet$Y; CatSet$X; CatSet$A

#Conditional search within a variable
CatSet$Y[CatSet$Y>35]
#Same, using the with function
with(CatSet,Y[Y>35])

#Defining a variable to be a factor
CatSet$A<-factor(CatSet$A)

#Looking ath the levels of a factor
levels(CatSet$A)

#Renaming the levels
levels(CatSet$A)<-c("A1","A2","A3")

#Listing new levels
with(CatSet,levels(A))

#Listing a subset using conditional index statements
with(CatSet,CatSet[Y>35,])
with(CatSet,CatSet[(X>0)&(X<55),c(2,3)])

#Save the updated data frame in an R source file
dump("CatSet",
   "C:/Docs/RCompanion/Chapter3/Data/Data03x02.R")

#Source command to recall CatSet when needed
source("C:/Docs/RCompanion/Chapter3/Data/Data03x02.R")

#Clear workspace
rm(list=ls(all=T))

#..................................................
# 3.5.2 Computations with Multivariate Datasets ...
#..................................................

#Create a matrix
X<-matrix(c(1,1,1,1,3,0,-1,2),nrow=4,ncol=2,byrow=F);X

#Obtain product of X with its transpose
XTX<-crossprod(X,X); XTX
t(X)%*%X

#Obtain inverse of the above product
XTX.Inv<-solve(XTX); XTX.Inv

#Test if two matrices are inverses of each other
round(crossprod(XTX,XTX.Inv))

#Clear workspace
rm(list=ls(all=T))

################################
# 3.6 Descriptive Statistics ###
################################

#..............................
# 3.6.1 Univariate datasets ...
#..............................

#Get data from a text file
UniVarSet<-read.table(
 "C:/Docs/RCompanion/Chapter3/Data/Data03x03.txt",header=T)
names(UniVarSet)

#Extract data
x<-UniVarSet$x; rm(UniVarSet)

#Explore contents
dim(x)
length(x)
mean(x)
var(x)
sd(x)
min(x)
max(x)
range(x)
median(x)
quantile(x,.25)
IQR(x)

#Obtain summaries
summary(x)
fivenum(x)

#................................
# 3.6.2 Multivariate datasets ...
#................................

#Clear workspace
rm(list=ls(all=T))

#Get data from text file
MultiNum<-read.table(
 "C:/Docs/RCompanion/Chapter3/Data/Data03x04.txt",header=T)

#Explore contents
names(MultiNum)
dim(MultiNum)
summary(MultiNum)

#Get variance of y
var(MultiNum$y)

#Get five-number-summary of y
with(MultiNum,fivenum(x2))

#Get variance-covariance matrix for x's and y
round(var(MultiNum),3)
round(cov(MultiNum),3)

#Get correlation coefficient matrix for x's and y 
round(cor(MultiNum),3)

#########################
# 3.7 For the curious ###
#########################

#Clear workspace
rm(list=ls(all=T))

#Recall dataset UniVarSet and extract data
UniVarSet<-read.table(
 "C:/Docs/RCompanion/Chapter3/Data/Data03x03.txt",header=T)
x<-UniVarSet$x; rm(UniVarSet)

#.......................................................
# 3.7.1 Constructing a grouped frequency distribution ..
#.......................................................

#Determine range of values
range(x)

#Set break points/partition
bpts<-seq(0,110,10)

#Create classes
class<-cut(x,breaks=bpts,include.lowest=T,right=F)

#Get class midpoints
mp<-seq(5,105,10)

#Get class frequencies
f<-summary(class)

#Get class relative frequencies
rf<-round(f/sum(f),3)

#Combine the lot by columns and check contents
freq.dist<-cbind("Mid Point"=mp,"Freq."=f,"Rel. Freq."=rf)
freq.dist

#Combine the lot in a data frame and check contents
freq.dist<-data.frame("Mid Point"=mp,"Freq."=f,"Rel. Freq."=rf)
freq.dist

#...........................................
# 3.7.2 Using a sink file to save output ...
#...........................................

sink(file="C:/Docs/RCompanion/Chapter3/Data/freqdist.txt",
                    append=F,type="output")
freq.dist
sink()
