bf.ttest <-
function(e,x,cutoff){
#Get test data and sorting data list names
dname<-deparse(substitute(e))
xname<-deparse(substitute(x))
#Partition list into two groups
e1<-cbind(x,e)[x<=cutoff,2]; e2<-cbind(x,e)[x>cutoff,2]
n<-length(x); n1<-length(e1); n2<-length(e2)
#Obtain group medians
e1.med<-median(e1); e2.med<-median(e2)
#Obtain deviations from medians
d1<-abs(e1-e1.med); d2<-abs(e2-e2.med)
#Obtain means of deviations from medians
d1.ave<-mean(d1); d2.ave<-mean(d2)
#Compute the pooled variance of deviations from the medians
p.var<-(sum((d1-d1.ave)^2)+
                  sum((d2-d2.ave)^2))/(n-2)
#Calculate the test statistic
tbf<-abs(d1.ave-d2.ave)/sqrt(p.var/n1+p.var/n2)
#Compute the p-value using the t-distribution
pv<-round(2*pt(-tbf,n-2),4)
#Store the results in a list
results<-list(statistic=c(t=tbf),parameters=c(df=n-2),
        p.value=pv, method=paste("Brown-Forsyth test ranked by",
          xname," and split at ",xname,"=",cutoff),data.name=dname,
                              alternative=c("Variances are unequal"))
#Format results as class hypothesis test and output
class(results)<-"htest";return(results)}

