######################## Chapter 7 Script #################
###########################################################

#Clear workspace
rm(list=ls(all=T))

######################
# 7.2 Improving Fit ##
######################

#............................................
# 7.2.1 Transforming explanatory variables ..
#............................................

######################################################
# Execute this command to duplicate section figures ##
######################################################

source("C:/Docs/RCompanion/Chapter7/Data/Data07x01.R")

#####################################################################
#Data generation code ..
#.......................

x<-runif(100,2,50)
y<-2-5/x+rnorm(100,mean=0,sd=.25)
XTransReg<-data.frame("x"=x,"y"=y)
#Plot to check for "niceness"
with(XTransReg,plot(x,y))
rm(x,y)

#Code to save, if so desired
dump("XTransReg","C:/Docs/RCompanion/Chapter7/Data/Data07x01.R")
#####################################################################

#...........
# Fig 7.1 ..
#...........

win.graph(width=4,height=3.5);par(ps=10,cex=.75)
with(XTransReg,plot(y~x))

#...........
# Fig 7.2 ..
#...........

noThink.mod<-lm(y~x,XTransReg)
win.graph(width=6,height=3.5);par(mfrow=c(1,2),ps=10,cex=.75)
with(noThink.mod,plot(fitted.values,residuals,
   		xlab="Fitted Values",ylab="Residuals"))
with(noThink.mod,qqnorm(residuals,main="QQ Plot of Residuals"))

#.........................
# Plot transformed data ..
#.........................

win.graph(width=4,height=3.5);par(ps=10,cex=.75)
with(XTransReg,plot(1/x,y))

#............................................
# Fit transformed model and plot residuals ..
#............................................

xTrans.mod<-lm(y~I(1/x),XTransReg)
win.graph(width=6,height=3.5);par(mfrow=c(1,2),ps=10,cex=.75)
with(xTrans.mod,plot(fitted.values,residuals,
   		xlab="Fitted Values",ylab="Residuals"))
with(xTrans.mod,qqnorm(residuals,main="QQ Plot of Residuals"))

#...........
# Fig 7.3 ..
#...........

win.graph(width=4,height=3.5);par(ps=10,cex=.75)
with(XTransReg,plot(1/x,abs(residuals(xTrans.mod)),ylab="|Residuals|"))

#.......................
# Brown-Forsyth test ...
#.......................

x<-XTransReg$x
source("C:/Docs/RCompanion/Chapter6/Functions/RegBFFTest.R")
with(xTrans.mod,bf.Ftest(residuals,1/x,0.025))

#.........................................................
# 7.2.2 Transforming explanatory and response variables ..
#.........................................................

#Clear workspace
rm(list=ls(all=T))

######################################################
# Execute this command to duplicate section figures ##
######################################################

source("C:/Docs/RCompanion/Chapter7/Data/Data07x02.R")

####################################################################
#Data generation code ..
#.......................

x<-runif(100,0,5)
y<-2*x/((1+rnorm(100,mean=1,sd=.1))*x+1)
#Plot to check for appearance
plot(y~x)
#Store in a data frame
XYTransReg<-data.frame("x"=x,"y"=y); rm(x,y)

#Save if so desired
dump("XYTransReg","C:/Docs/RCompanion/Chapter7/Data/Data07x02")
#####################################################################

#...........
# Fig 7.4 ..
#...........

win.graph(width=6.5,height=2.5)
par(mfrow=c(1,3),ps=10,cs=.75)
attach(XYTransReg)
plot(x,y,sub="(a)");plot(1/x,y,sub="(b)")
plot(1/x,1/y,sub="(c)")
detach(XYTransReg)

#.....................................................
# Fit transformed model and obtain diagnostic plots ..
#.....................................................

xyTrans.mod<-lm(1/y~I(1/x),XYTransReg)
win.graph(width=6,height=3.5);par(mfrow=c(1,2),ps=10,cex=.75)
with(xyTrans.mod,plot(fitted.values,residuals,
   		xlab="Fitted Values",ylab="Residuals"))
with(xyTrans.mod,qqnorm(residuals,main="QQ Plot of Residuals"))



####################################
# 7.3 Normalizing Transformations ## 
####################################

#Clear workspace
rm(list=ls(all=T))

#.......................................................
# 7.3.2 Power (or log) transformations by observation ..
#.......................................................

######################################################
# Execute this command to duplicate section figures ##
######################################################

source("C:/Docs/RCompanion/Chapter7/Data/Data07x03.R")

####################################################################
# Data generation code ..
#........................

x<-seq(0,2,.04)
y<-(x+rnorm(50,mean=2,sd=0.05))^(5/2)
RightSkewed<-data.frame("x"=x,"y"=y); rm(x,y)

#Save if so desired
dump("RightSkewed","C:/Docs/RCompanion/Chapter7/Data/Data07x03.R")
####################################################################

#................................................................................
# Fit untransformed model and extract standardised residuals and fitted values ..
#................................................................................

RS.mod<-lm(y~x,RightSkewed)
r<-residuals(RS.mod)/summary(RS.mod)$sigma
f<-fitted.values(RS.mod)

#...........
# Fig 7.5 ..
#...........

win.graph(width=6.5,height=2.5);par(mfrow=c(1,3),ps=10,cex=.75)
with(RightSkewed,plot(y~x,sub="(a)"))
qqnorm(r,ylab="Standardized Residuals",
                   main=NULL,sub="(b)")
abline(a=0,b=1,lty=3)
plot(r~f,xlab="Fitted Values",
    ylab="Standardized Residuals",sub="(c)")
abline(h=0)
par(mfrow=c(1,1))

#Shapiro-Wilk test for normality
shapiro.test(r)

#.....................................
# Fit square root transformed model ..
#.....................................

trans1.mod<-lm(sqrt(y)~x,RightSkewed)
r<-residuals(trans1.mod)/summary(trans1.mod)$sigma
f<-fitted.values(trans1.mod)

#......................................................
# Diagnostic plots for square root transformed model ..
#......................................................

win.graph(width=6.5,height=2.5)
par(mfrow=c(1,3),ps=10,cex=.75)
with(RightSkewed,plot(sqrt(y)~x,ylab=expression(sqrt(y))))
qqnorm(r,ylab="Standardized Residuals",main="QQ Plot")
title(sub="Square root Transformed model")
abline(a=0,b=1,lty=3)
plot(r~f,xlab="Fitted Values",ylab="Standardized Residuals")
title(sub="Square root Transformed model")
abline(h=0)
par(mfrow=c(1,1))

#Shapiro-Wilk test for normality
shapiro.test(r)

#...............................
# Fit y^0.4 transformed model ..
#...............................

trans2.mod<-lm(y^0.4~x,RightSkewed)
r<-residuals(trans2.mod)/summary(trans2.mod)$sigma
f<-fitted.values(trans2.mod)

#...........
# Fig 7.6 ..
#...........

win.graph(width=6.5,height=2.5)
par(mfrow=c(1,3),ps=10,cex=.75)
with(RightSkewed,plot(y^0.4~x,ylab=expression(y^{0.4}),sub="(a)"))
qqnorm(r,ylab="Standardized Residuals",main=NULL,sub="(b)")
abline(a=0,b=1,lty=3)
plot(r~f,xlab="Fitted Values",ylab="Standardized Residuals",sub="(c)")
abline(h=0); par(mfrow=c(1,1))

#Shapiro-Wilk test for normality
shapiro.test(r)

#..............................
# 7.3.3 The Box-Cox procedure ..
#..............................

#...........
# Fig 7.7 ..
#...........

library(MASS)
win.graph(width=6,height=3)
par(mfrow=c(1,2),ps=10,cex=.75)
boxcox(RS.mod)
boxcox(RS.mod,lambda=seq(0.15,0.45,0.05))
axis(1,at=seq(0.15,0.45,.01),tcl=.25,labels=F)
detach(package:MASS)

#Compare coefficients for equivalent models
lm((y^(.3)-1)/.3~x,RightSkewed)
lm(y^(.3)~x,RightSkewed)

#...........
# Fig 7.8 ..
#...........
trans3.mod<-lm(y^(.3)~x,RightSkewed)
r<-residuals(trans3.mod)/summary(trans3.mod)$sigma
f<-fitted.values(trans3.mod)

win.graph(width=6.5,height=2.5)
par(mfrow=c(1,3),ps=10,cex=.75)
with(RightSkewed,plot(y^0.3~x,ylab=expression(y^{0.3}),sub="(a)"))
qqnorm(r,ylab="Standardized Residuals",main=NULL,sub="(b)")
abline(a=0,b=1,lty=3)
plot(r~f,xlab="Fitted Values",ylab="Standardized Residuals",sub="(c)")
abline(h=0); par(mfrow=c(1,1))

#Shapiro-Wilk test for normality
shapiro.test(r)

#############################################
# 7.4 Variance Stabilizing Transformations ##
#############################################

#Clear woskspace
rm(list=ls(all=T))

#........................
# Code for Figure 7.9 ...
#........................

#Generate generic xy-pairs having a linear fit
x<-runif(200,1,3)
y<-(1+x+rnorm(200,mean=0,sd=0.25))

#Set a graphics window to hold nine plots
win.graph(width=6.5,height=8)
par(mfrow=c(3,3),ps=10,cex=.75)

#Plot (a)
i<--1.5
z<-y^(1/i)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k*hat(y)^{5/2})),
		sub=expression(paste("(a) Use ",z," = ",y^{-3/2})))
	xv<-seq(min(fitted.values(model.fit)),max(fitted.values(model.fit)),
	(max(fitted.values(model.fit))-min(fitted.values(model.fit)))/100)
	yv<-xv^((1-i))/3.15
	lines(xv,yv,lty=5)

#Plot (b)
i<--1
z<-y^(1/i)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k*hat(y)^{2})),
		sub=expression(paste("(b) Use ",z," = ",y^{-1})))
	xv<-seq(min(fitted.values(model.fit)),max(fitted.values(model.fit)),
	(max(fitted.values(model.fit))-min(fitted.values(model.fit)))/100)
	yv<-xv^((1-i))/2
	lines(xv,yv,lty=5)

#Plot (c)
i<--.5
z<-y^(1/i)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k*hat(y)^{3/2})),
		sub=expression(paste("(c) Use ",z," = ",y^{-1/2})))
	xv<-seq(min(fitted.values(model.fit)),max(fitted.values(model.fit)),
	(max(fitted.values(model.fit))-min(fitted.values(model.fit)))/100)
	yv<-xv^((1-i))
	lines(xv,yv,lty=5)

#Plot (d)
z<-exp(y)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k*hat(y))),
		sub=expression(paste("(d) Use ",z," = ",log(y))))
	xv<-seq(min(fitted.values(model.fit)),max(fitted.values(model.fit)),
	(max(fitted.values(model.fit))-min(fitted.values(model.fit)))/100)
	yv<-xv/2
	lines(xv,yv,lty=5)

#Plot (e)
i=.5
z<-y^(1/i)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k*hat(y)^{1/2})),
		sub=expression(paste("(e) Use ",z," = ",y^{1/2})))
	xv<-seq(min(fitted.values(model.fit)),max(fitted.values(model.fit)),
	(max(fitted.values(model.fit))-min(fitted.values(model.fit)))/100)
	yv<-1.1*xv^((1-i))
	lines(xv,yv,lty=5)

#Plot (f)
i=1
z<-y^(1/i)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k)),
		sub=expression(paste("(f) No transformation, use ",z," = ",y)))
	abline(h=.5,lty=5)

#Plot (g)
i=1.5
z<-y^(1/i)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k*hat(y)^{-1/2})),
		sub=expression(paste("(g) Use ",z," = ",y^{3/2})))
	xv<-seq(min(fitted.values(model.fit)),max(fitted.values(model.fit)),
	(max(fitted.values(model.fit))-min(fitted.values(model.fit)))/100)
	yv<-xv^((1-i))/2.9
	lines(xv,yv,lty=5)

#Plot (h)
i=2
z<-y^(1/i)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k*hat(y)^{-1})),
		sub=expression(paste("(h) Use ",z," = ",y^{2})))
	xv<-seq(min(fitted.values(model.fit)),max(fitted.values(model.fit)),
	(max(fitted.values(model.fit))-min(fitted.values(model.fit)))/100)
	yv<-xv^((1-i))/4
	lines(xv,yv,lty=5)

#Plot (i)
i=2.5
z<-y^(1/i)
      model.fit<-lm(z~x)
	plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		main=expression(paste("|",hat(epsilon),"|< ",k*hat(y)^{-3/2})),
		sub=expression(paste("(i) Use ",z," = ",y^{5/2})))
	xv<-seq(min(fitted.values(model.fit)),max(fitted.values(model.fit)),
	(max(fitted.values(model.fit))-min(fitted.values(model.fit)))/100)
	yv<-xv^((1-i))/5
	lines(xv,yv,lty=5)
par(mfrow=c(1,1))

#.........................
# Code for Figure 7.10 ...
#.........................

#Clear woskspace
rm(list=ls(all=T))

#Generate data
x1<-runif(200,0,pi/2)
y<-sin(x1+rnorm(200,mean=0,sd=0.25))^2

#Plot graph
win.graph(width=3,height=3); par(ps=10,cex=.75)
model.fit<-lm(y~x1)
plot(fitted.values(model.fit),abs(residuals(model.fit)),
		xlab=expression(hat(y)),ylab=expression(paste("| ",hat(epsilon)," |")),las=1,
		xlim=c(0,1.1),main=expression(paste("|",hat(epsilon),"|< ",k*sqrt(hat(y)*(1-hat(y))))),
		sub=expression(paste("Use ",z," = ",arcsin(sqrt(y)))))
xv<-seq(0,1,.1)
yv<-1.6*xv*(1-xv)
lines(xv,yv,lty=5)

##############################
# 7.5 Polynomial Regression ##
##############################

#Clear workspace
rm(list=ls(all=T))

######################################################
# Execute this command to duplicate section figures ##
######################################################

source("C:/Docs/RCompanion/Chapter7/Data/Data07x04.R")

##################################################################
# Data generation code ..
#........................

x<-runif(50,0,5); y<-round(-2*x^2+9*x+4+rnorm(50,0,1),1)
PolyReg<-data.frame("x"=x,"y"=y); rm(x,y)

#Save if so desired
dump("PolyReg","C:/Docs/RCompanion/Chapter7/Data/Data07x04.R")
###################################################################

#Fit the quadratic model
quad.mod<-lm(y~x+I(x^2),PolyReg)

#Obtain relevant diagnostics material
quad.sum<-summary(quad.mod)
e<-residuals(quad.mod)
y.fit<-fitted.values(quad.mod)
s<-quad.sum$sigma
r<-e/s

#............
# Fig 7.11 ..
#............

win.graph(width=6.5,height=6.5)
par(mfrow=c(2,2),ps=10,cex=.75)
with(PolyReg,plot(y~x,sub="(a)"))
qqnorm(r,ylab="Standardized Residuals",main=NULL,sub="(b)")
abline(a=0,b=1,lty=3)
plot(r~y.fit,xlab="Fitted Values",ylab="Standardized Residuals",sub="(c)")
abline(h=0)
with(PolyReg,plot(r~x,ylab="Standardized Residuals",sub="(d)"))
abline(h=0)
par(mfrow=c(1,1))

#Check model fit
summary(quad.mod)

#............
# Fig 7.12 ..
#............

win.graph(width=3,height=3);par(ps=10,cex=.75)
xvals<-seq(0,5,.1); yvals<-4.445+8.527*xvals-1.939*xvals^2
with(PolyReg,plot(y~x)); lines(xvals,yvals)



#################################
# 7.6 Piecewise Defined Models ##
#################################

#Clear workspace
rm(list=ls(all=T))

######################################################
# Execute this command to duplicate section figures ##
######################################################

source("C:/Docs/RCompanion/Chapter7/Data/Data07x05.R")

####################################################################
# Data generation code ..
#........................

x1<-runif(40,0,1); x2<-runif(40,1,3)
y1<--10*x1^2+10*x1+1+rnorm(40,mean=0,sd=.25)
y2<-x2+rnorm(40,mean=0,sd=.25)
x<-c(x1,x2); y<-c(y1,y2)
TwoPiece<-data.frame("x"=x,"y"=y);rm(x,y,x1,x2,y1,y2)

#Save if so desired
dump("TwoPiece","C:/Docs/RCompanion/Chapter7/Data/Data07x05.R")
####################################################################

#............................
# 7.6.1 Subset regression ...
#............................

#Fit the quadratic piece
piece1<-lm(y~x+I(x^2),TwoPiece,subset=(x<1))
#Fit the linear piece
piece2<-lm(y~x,TwoPiece,subset=(x>=1))

#............
# Fig 7.13 ..
#............

#Set window parameters
win.graph(width=4,height=3); par(ps=10,cex=.75)
#Plot the data and a vertical line at x=1
plot(y~x,TwoPiece,xlim=c(0,3),ylim=c(0,4.5))
abline(v=1,lty=3)
#Get coefficient estimates
a<-piece1$coefficients; b<-piece2$coefficients
#Plot piece 1
xv<-seq(0,1,.01); yv<-a[1]+a[2]*xv+a[3]*xv^2; lines(xv,yv)
#Plot piece 2
segments(1,b[1]+b[2]*1,3,b[1]+b[2]*3)

#.........................................
# 7.6.2 Continuous piecewise regression ..
#.........................................

#............
# Fig 7.14 ..
#............

#Define basis functions
bsl<-function(x) ifelse(x<1,1-x,0)
bsr<-function(x) ifelse(x<1,0,x-1)
#Fit the model
bs.mod<-lm(y~bsl(x)+I(bsl(x)^2)+bsr(x),TwoPiece)

#Plot data and fitted model
win.graph(width=4,height=3); par(ps=10,cex=.75)
plot(y~x,TwoPiece,xlim=c(0,3),ylim=c(0,5))
abline(v=1,lty=3)
b<-bs.mod$coefficients
xv<-seq(0,3,.01);yv<-b[1]+b[2]*bsl(xv)+b[3]*bsl(xv)^2+b[4]*bsr(xv)
lines(xv,yv)

#...............................
# 7.6.3 A three piece example ..
#...............................

#Clear workspace
rm(list=ls(all=T))

######################################################
# Execute this command to duplicate section figures ##
######################################################

source("C:/Docs/RCompanion/Chapter7/Data/Data07x06.R")

#####################################################################
# Data generation code ..
#........................

x1<-runif(30,0,1); x2<-runif(30,1,2); x3<-runif(30,2,3)
y1<-5+2*x1+rnorm(30,mean=0, sd=.1)
y2<-7-(x2-1)+rnorm(30,mean=0, sd=.1)
y3<-6+1/2*(x3-2)+rnorm(30,mean=0, sd=.1)
x<-c(x1,x2,x3); y<-c(y1,y2,y3)
ThreePiece<-data.frame(cbind(x,y)); rm(x1,x2,x3,y1,y2,y3,x,y)

#Save if so desired
dump("ThreePiece","C:/Docs/RCompanion/Chapter7/Data/Data07x06.R")
####################################################################

#............
# Fig 7.15 ..
#............

#Fit the model
bs1<-function(x) {ifelse(x<1,1-x,0)}
bs2<-function(x) {ifelse(x<1,0,ifelse(x<2,x-1,1))}
bs3<-function(x) {ifelse(x<2,0,2-x)}
bs3.mod<-lm(y~bs1(x)+bs2(x)+bs3(x),ThreePiece)

#Plot the graph
win.graph(width=4,height=3); par(ps=10,cex=.75)
b<-bs3.mod$coefficients
with(ThreePiece,plot(x,y))
xv<-seq(0,3,.01); yv<-b[1]+b[2]*bs1(xv)+b[3]*bs2(xv)+b[4]*bs3(xv)
lines(xv,yv)

##########################################
# 7.7 Introducing Categorical variables ##
##########################################

#.......................................
# 7.7.1 Paralell straight-line models ..
#.......................................

#Clear workspace
rm(list=ls(all=T))

######################################################
# Execute this command to duplicate section figures ##
######################################################

source("C:/Docs/RCompanion/Chapter7/Data/Data07x07.R")

##############################################################
# Data generation code ..
#........................

x<-runif(30,0,2); y1<-1+5*x+rnorm(30,sd=.35)
y2<-3+5*x+rnorm(30,sd=.35); y3<-5+5*x+rnorm(30,sd=.35)
x<-c(x,x,x); y<-c(y1,y2,y3)
Category<-c(rep(1,30),rep(2,30),rep(3,30))
CatReg1<-data.frame(x,y,Category); rm(x,y1,y2,y3,y,Category)
CatReg1$Category<-factor(CatReg1$Category)

#Save if so desired
dump("CatReg1","C:/Docs/RCompanion/Chapter7/Data/Data07x07.R")
##############################################################
#.............
# Fig 7.16 ...
#.............

win.graph(width=4,height=3.5); par(mfrow=c(1,1),ps=10,cex=.75)
with(CatReg1,plot(x,y,pch=unclass(Category)))

#Partial F-test to test parallel lines assumption
parallel.mod<-lm(y~x+Category,CatReg1)
nonpar.mod<-lm(y~x+Category+x:Category,CatReg1)
anova(parallel.mod,nonpar.mod)

parallel.mod

#............
# Fig 7.17 ..
#............

win.graph(width=4,height=3.5); par(mfrow=c(1,1),ps=10,cex=.75)
with(CatReg1,plot(x,y,pch=unclass(Category)))

#Extract parameter estimates
bs<-parallel.mod$coefficients
#Line for level 1
abline(a=bs[1],b=bs[2],lty=1)
#Line for level 2
abline(a=bs[1]+bs[3],b=bs[2],lty=2)
#Line for level 3
abline(a=bs[1]+bs[4],bs[2],lty=3)
#Include legend
legend(0,16,bty="n",
   legend=c("Level 1","Level 2","Level 3"),lty=c(1,2,3))

#............................................
# 7.7.2 Non-parallel straight-line models ...
#............................................

#Clear workspace
rm(list=ls(all=T))

######################################################
# Execute this command to duplicate section figures ##
######################################################

source("C:/Docs/RCompanion/Chapter7/Data/Data07x08.R")

##############################################################
# Data generation code ..
#........................

x<-runif(30,0,2)
y1<-1+5*x+rnorm(30,sd=.25)
y2<-3+3*x+rnorm(30,sd=.25)
y3<-5+2*x+rnorm(30,sd=.25)
x<-c(x,x,x)
y<-c(y1,y2,y3)
Category<-c(rep(1,30),rep(2,30),rep(3,30))
CatReg2<-data.frame(x,y,Category); rm(x,y1,y2,y3,y,Category)
CatReg2$Category<-factor(CatReg2$Category)

#Save if so desired
dump("CatReg2","C:/Docs/RCompanion/Chapter7/Data/Data07x08.R")
##############################################################

#Partial F-test to test parallel lines assumption
parallel.mod<-lm(y~x+Category,CatReg2)
nonpar.mod<-lm(y~x+Category+x:Category,CatReg2)
anova(parallel.mod,nonpar.mod)

round(nonpar.mod$coefficients,3)

#............
# Fig 7.18 ..
#............

win.graph(width=4,height=3.5); par(mfrow=c(1,1),ps=10,cex=.75)
with(CatReg2,plot(x,y,pch=unclass(Category)))
bs<-nonpar.mod$coefficients
abline(a=bs[1],b=bs[2],lty=1)
abline(a=bs[1]+bs[3],b=bs[2]+bs[5],lty=2)
abline(a=bs[1]+bs[4],b=bs[2]+bs[6],lty=3)
legend(0,11,bty="n",
   legend=c("Level 1","Level 2","Level 3"),lty=c(1,2,3))

########################
# 7.8 For the Curious ##
########################

#.....................................
# 7.8.1 The Box-Cox procedure revisited ..
#.....................................

#Clear workspace
rm(list=ls(all=T))

#Recall dataset RightSkewed
source("C:/Docs/RCompanion/Chapter7/Data/Data07x03.R")

#For convenience
attach(RightSkewed)
#Define transformation function
z<-function(Y,a){
	if(abs(a)<=0.001){log(Y)} else {(Y^a-1)/a}}
#Obtain sample size and sequence of lambdas
n<-length(y); l<-seq(.15,.45,0.01)
#Initialize likelihood function vector
L<-numeric(length(l))
#Compute likelihood values
for (k in 1:length(l)){
	e<-residuals(lm(z(y,l[k])~x))
	L[k]<- -n/2*log(sum(e^2)/n)+(l[k]-1)*sum(log(y))}
#Set up window for plotting
win.graph(width=4,height=3)
par(mfrow=c(1,1),ps=10,cex=.75)
#Obtain plot of likelihood function against lambda
plot(L~l,type="l",xlab=expression(lambda),
		ylab=expression(l(lambda)))
#Mark maximizing lambda
abline(v=l[L==max(L)],lty=3)
#Insert maximizing lambda value on figure
text(max(l),max(L),
	bquote(hat(lambda)==.(round(l[L==max(L)],2))),adj=1)
detach(RightSkewed)

#Run function boxcox again for comparison
RS.mod<-lm(y~x,RightSkewed)
library(MASS)
win.graph(width=4,height=3)
par(mfrow=c(1,1),ps=10,cex=.75)
boxcox(RS.mod,lambda=seq(0.15,0.45,0.01),interp=F)
axis(1,at=seq(0.15,0.45,.01),tcl=.25,labels=F)
detach(package:MASS)


#......................................................
# 7.8.2 Inserting mathematical annotation in figures ..
#......................................................

# Note: this is a duplicate of code for Figure 7.10
#Additional comments are provided here

#The first two lines generate the data
x<-runif(200,0,pi/2)
y<-sin(x+rnorm(200,mean=0,sd=0.25))^2
#Fit the data to a model
mod<-lm(y~x)
#Plot the points, set tick mark label orientation
with(mod,plot(fitted.values,abs(residuals),las=1,
#Insert the main title with mathematical annotation
 main=expression(paste(abs(hat(epsilon))<
                  k*sqrt(hat(y)*(1-hat(y))))),
#Set x-limits and insert x-label
  xlim=c(0,1.1),xlab=expression(hat(y)),
#Insert the y-label and subtitle, both with mathematical annotation
   ylab=expression(paste(abs(hat(epsilon)))),
    sub=expression(paste("Use ",z==arcsin(sqrt(y))))))
#Plot the bounding curve (eye-balling the scale)
xv<-seq(0,1,.01); yv<-1.6*xv*(1-xv); lines(xv,yv,lty=5)

#............................
# 7.8.3 The split function ..
#............................

#Clear workspace
rm(list=ls(all=T))

#Recall dataset CatReg1
source("C:/Docs/RCompanion/Chapter7/Data/Data07x07.R")

win.graph(width=4,height=3.5); par(mfrow=c(1,1),ps=10,cex=.75)
with(CatReg1,plot(x,y,type="n"))
xs<-with(CatReg1,split(x,Category))
ys<-with(CatReg1,split(y,Category))
for (i in 1:3){points(xs[[i]],ys[[i]],pch=i)}


