######################## Chapter 9 Script #################
###########################################################

######################
# 9.1 Introduction ###
######################

#Clear workspace
rm(list=ls(all=T))

##############################################################
#Execute this command to duplicate contents of this chapter ##
##############################################################

source("C:/Docs/RCompanion/Chapter8/Data/Data08x01.R")

###########################################
# 9.2 Detection of structural violations ##
###########################################

#..............................
# 9.2.1 Matrix scatter plots ..
#..............................

#...........
# Fig 9.1 ..
#...........

#Option A
win.graph(width=6,height=6);par(ps=10,cex=.75)
plot(MultipleReg)

#Option B
win.graph(width=6,height=6);par(ps=10,cex=.75)
with(MultipleReg,pairs(cbind(y,x1,x2,x3)))

#................................
# 9.2.2 Partial residual plots ..
#................................

#...........
# Fig 9.2 ..
#...........

#Fit the model
Multiple.mod<-lm(y~x1+x2+x3,MultipleReg)
#Open a graphics window with desired properties
win.graph(width=6,height=2.5)
par(mfrow=c(1,3),ps=10,cex=.75)
#Load package faraway and plot the graphs
require(faraway)
for (i in 1:3) {prplot(Multiple.mod,i)}
#Detach package faraway and reset parameter mfrow
detach(package:faraway)
par(mfrow=c(1,1))

#.................................
# 9.2.3 Testing for lack of fit ..
#.................................

##############################################################
#Execute this command to duplicate contents of this section ##
##############################################################

source("C:/Docs/RCompanion/Chapter9/Data/Data09x01.R")

###############################################################
# Data generation code ..
#........................

x<-round(runif(50,0,4),1)
#Insert some replicates
x<-c(x,x[rbinom(50,4,.5)+1])
#Generate some "observed" responses
y<-round((9-2*x)^x+rnorm(length(x),0,1),1)+4
#Plot to look at data
plot(y~x)
#Store in a data frame
LOFData<-data.frame(y,x)
#Tidy up some
rm(y,x)

#Save, if so desired
dump("LOFData","C:/Docs/RCompanion/Chapter9/Data/Data09x01.R")
###############################################################
#...........
# Fig 9.3 ..
#...........

win.graph(width=3,height=3); par(ps=10,cex=.75)
with(LOFData,plot(x,y))

reduced.mod<-lm(y~x,LOFData)
full.mod<-lm(y~factor(x),LOFData)
anova(reduced.mod,full.mod)

ls()
rm(i,full.mod,reduced.mod,LOFData)

##########################
# 9.3 Multicollinearity ##
##########################

#Get vifs
rXX<-with(MultipleReg,cor(cbind(x1,x2,x3)))
rXX.inv<-solve(rXX)
vifs<-numeric(dim(rXX.inv)[1]);vars<-NULL
for (i in 1:3){vifs[i]<-rXX.inv[i,i]
		   vars<-c(vars,paste("x",i,sep=""))}
names(vifs)<-vars

#..................................
# 9.3.1 Correlation coefficients ..
#..................................

round(rXX,3)

#....................................
# 9.3.2 Variance inflation factors ..
#....................................

vifs

###########################
# 9.4 Variable Selection ##
###########################

#...........................................................
# 9.4.1 Contribution significance of individual variables ..
#...........................................................

#Coefficients summary
Multiple.sum<-summary(Multiple.mod)
round(Multiple.sum$coefficients,4)

#..........................................................
# 9.4.2 Contribution significance of groups of variables ..
#..........................................................

#Partial F-tests
red.mod<-lm(y~x1+x3,MultipleReg)
anova(red.mod,Multiple.mod)

red.mod<-lm(y~x2,MultipleReg)
anova(red.mod,Multiple.mod)

#........................
# 9.4.3 Drop1 function ..
#........................

drop1(Multiple.mod,~x1+x3,test="F")

#.......................
# 9.4.4 Add1 function ..
#.......................

add1(Multiple.mod,~.+I(x1*x2),test="F")

add1(Multiple.mod,~x1+x2+x3+I(x1*x2),test="F")

#.......................................
# 9.4.5 Stepwise selection algorithms ..
#.......................................

step(Multiple.mod)

step(Multiple.mod,test="F")

##################################
# 9.5 Model selection criteria ###
##################################

require(leaps)
#Use the function regsubsets to get details on all possible models
search.results<-regsubsets(y~x1+x2+x3,data=MultipleReg,
                            method="exhaustive",nbest=7)
#Extract search criteria from the object search.results
selection.criteria<-summary(search.results)
detach(package:leaps)

names(selection.criteria)

#Obtain sample size
n<-length(MultipleReg[,1])
#Extract the number of variables included in the test model
q<-as.integer(row.names(selection.criteria$which))
#Extract the mean square error for the test model
mse<-selection.criteria$rss/(n-q-1)
#Extract R square and adjusted R square
R.sq<-selection.criteria$rsq
AdjR.sq<-selection.criteria$adjr2
#Extract Mallow's statistic
Cp<-selection.criteria$cp
#Compute the AIC for the test model by formula
aic.f<-n*log(selection.criteria$rss)-n*log(n)+2*(q+1)
#Compute the BIC for the test model by formula
bic.f<-n*log(selection.criteria$rss)-n*log(n)+(q+1)*log(n)
#Extract variable information
var<-as.matrix(selection.criteria$which[,2:4])
#Create the criteria table
criteria.table<-data.frame(cbind(q,mse,R.sq,AdjR.sq,Cp,aic.f,bic.f,
var[,1],var[,2],var[,3]),row.names=NULL)
#Name the columns
names(criteria.table)<-c("q","MSE","Rsq","aRsq","Cp","AIC","BIC","x1","x2","x3")
#Clean up the workspace
rm(n,q,mse,R.sq,AdjR.sq,Cp,aic.f,bic.f,var)
#Take a look at the contents of the criteria table
round(criteria.table,2)

#...........
# Fig 9.4 ..
#...........

win.graph(width=4,height=3.5)
par(ps=10,cex=.75)
stuff<-with(criteria.table,criteria.table[Cp<=300,])
with(stuff,stripchart(Cp~factor(q+1),vertical=T))
text(1.1,50,"Model-13");text(1.1,210,"Model-23")
text(1.98,4,"Model-123",adj=1)
title(xlab="q+1",sub="Number of parameters")

#.........................
# 9.5.6 PRESS statistic ..
#.........................

mods<-c("y~x1+x3","y~x2+x3","y~x1+x2+x3")
P<-numeric(3)
for (i in 1:3){mod<-lm(mods[i],MultipleReg)
		   r<-residuals(mod)
		   h<-hatvalues(mod)
		   d<-r/(1-h)
		   P[i]<-sum(d^2)}
P<-data.frame("PRESS"=P,row.names=mods); P

library(MPV)
mods<-c("y~x1+x3","y~x2+x3","y~x1+x2+x3")
P<-numeric(3)
for (i in 1:3){mod<-lm(mods[i],MultipleReg)
		   P[i]<-PRESS(mod)}
P<-data.frame("PRESS"=P,row.names=mods); P
detach(package:MPV)

########################
# 9.6 For the curious ##
########################

#................................
# 9.6.1.2 The condition number ..
#................................

#Eigenvalues
(eigenvals<-eigen(rXX)$values)
sum(eigenvals)

#Condition number
(c.number<-max(eigenvals)/min(eigenvals))

#.............................
# 9.6.1.3 Condition indices ..
#.............................

#Condition indices
(c.indices<-max(eigenvals)/eigenvals)

(eigenvecs<-eigen(rXX)$vectors)

#...............................
# 9.6.2 Variance proportions ...
#...............................

attach(MultipleReg)
x0<-rep(1,40) 
Z<-cbind(x0/sqrt(sum(x0^2)),x1/sqrt(sum(x1^2)),
		x2/sqrt(sum(x2^2)),x3/sqrt(sum(x3^2)))
detach(MultipleReg)
ZTZ<-crossprod(Z,Z)

l<-eigen(ZTZ)$values; v<-eigen(ZTZ)$vectors
ci<-max(l)/l

sum(l)

#Initialize objects for ak and pjk
a<-numeric(4); p<-cbind(a,a,a,a)
colnames(p)<-c("b0","b1","b2","b3")
#Generate variance proportions
for (k in 1:4){
a[k]<-sum(v[k,]^2/l)
for (j in 1:4){
	p[j,k]<-(v[k,j]^2/l[j])/a[k]}}
#Store the results in a data frame
collinearity<-data.frame("eigenval."=l,"Cond.Index"=ci,p)
round(collinearity,3)

#............................................
# 9.6.3 Model selection criteria revisited ..
#............................................

#Prepare "switches" for each variable, 1 says include, 0 says no
x1<-c(1,0,0,1,1,0,1)
x2<-c(0,1,0,1,0,1,1)
x3<-c(0,0,1,0,1,1,1)
#Initialize the model formula storage object
mods<-character(7)
#Use a for loop to generate each model formula
for (i in 1:7){
    mods[i]<-paste("y~",
      ifelse(x1[i]==1,"x1",""),
        ifelse((x1[i]!=0)&((x2[i]!=0)|(x3[i]!=0)),"+",""),
         ifelse(x2[i]==1,"x2",""),
            ifelse((x2[i]==1)&(x3[i]!=0),"+",""),
              ifelse(x3[i]==1,"x3",""),sep="")}

#Check contents of mods
mods

#Variable counter
q<-integer(7)
#Residual sum of squares and mean square error
SSE<-numeric(7); MSE<-numeric(7)
#R square and adjusted R square
Rsq<-numeric(7); A.Rsq<-numeric(7)
#Mallow's statistic and PRESS statistic
Cp<-numeric(7); P<-numeric(7)
#AIC and BIC
aic<-numeric(7); bic<-numeric(7)

#Get sample size and mean square error for full model
n<-dim(MultipleReg)[1]; s.sq<-Multiple.sum$sigma^2
#Begin for loop
for (i in 1:7){
     #fit model i and get summary
     mod<-lm(mods[i],MultipleReg)
     mod.sum<-summary(mod)
     #Get number of variables
     q[i]<-as.integer(x1[i]+x2[i]+x3[i])
     #Get mean square error and residual sum of squares
     MSE[i]<-round(mod.sum$sigma^2,2)
     SSE[i]<-mod.sum$sigma^2*(n-q[i]-1)
     #Get R square and adjusted R square
     Rsq[i]<-round(mod.sum$r.squared,3)
     A.Rsq[i]<-round(mod.sum$adj.r.squared,3)
     #Get Mallow's statistic
     Cp[i]<-round(SSE[i]/s.sq+2*(q[i]+1)-n)
     #Get AIC and BIC
     aic[i]<-round(n*log(SSE[i])-n*log(n)+2*(q[i]+1))
     bic[i]<-round(n*log(SSE[i])-n*log(n)+log(n)*(q[i]+1))
     #Get PRESS
     r<-residuals(mod); h<-hatvalues(mod)
     P[i]<-round(sum((r/(1-h))^2))}

#Create a data frame
Model.selection<-data.frame(
#Put in q, MSE, R squared and adjusted R squared
"q"=q, "MSE"=MSE, "R.Sq"=Rsq, "A.Rsq"=A.Rsq,
#Put in Cp, AIC, BIC and PRESS
"Cp"=Cp, "AIC"=aic, "BIC"=bic, "PRESS"=P,
#Assign row names
row.names=mods)

Model.selection



